/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "roundCorner.h"
#include "../newtEngine/scene.h"


RoundCornerMesh::RoundCornerMesh(float radius, float phiAngle, int nbPhi, float phiTexBegin, float phiTexEnd,
								 float lambdaAngle, int nbLambda, float lambdaTexBegin, float lambdaTexEnd, bool inside):
	Mesh("roundCorner", false, true, false, true), mRadius(radius), mPhiAngle(phiAngle), mPhiTexBegin(phiTexBegin), mPhiTexEnd(phiTexEnd),
	mLambdaAngle(lambdaAngle), mLambdaTexBegin(lambdaTexBegin), mLambdaTexEnd(lambdaTexEnd), mInside(inside) {
		
		
	float meshLevel = scene->getGraphicOptions()->getMeshLevel();
	float value;
		
	value = (float) nbPhi * meshLevel;
	if(value < 3) {
		mNbPhi = 3;
	} else {
		mNbPhi = (int) value;
	}

	value = (float) nbLambda * meshLevel;
	if(value < 3) {
		mNbLambda = 3;
	} else {
		mNbLambda = (int) value;
	}			
}

RoundCornerMesh::~RoundCornerMesh() {
}


		
void RoundCornerMesh::initFirstParams() {
	geometry.nbVertices = (mNbPhi + 1) * ( mNbLambda + 1);
	
	geometry.nbIndexArrays = mNbLambda;
	geometry.primType = new GLenum[mNbLambda];
	for(int i=0;i<mNbLambda;i++) {
		geometry.primType[i] = GL_QUAD_STRIP;
	}
	
	geometry.nbIndices = new int[mNbLambda];
	for(int i=0;i<mNbLambda;i++) {
		geometry.nbIndices[i] = 2 * (mNbPhi+1);
	}	
}
		
void RoundCornerMesh::createVertices(Vector3 *ptr) {
	float lambda, phi;
	float lambdaMin = M_PI /2. - mLambdaAngle;	
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/mNbLambda)*mLambdaAngle + lambdaMin; 
		for(int j=0;j<=mNbPhi;j++) {
			phi = ((float) j/mNbPhi - 1./2.)*mPhiAngle; 
			*ptr = mRadius * Vector3(cos(lambda)*sin(phi),sin(lambda),cos(lambda)*cos(phi));
	    	ptr ++;
		}
	}
}

void RoundCornerMesh::createColors(Color *ptr) {

}

void RoundCornerMesh::createNormals(Vector3 *ptr) {
	float lambda, phi;
	float lambdaMin = M_PI /2. - mLambdaAngle;	
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/mNbLambda)*mLambdaAngle + lambdaMin; 
		for(int j=0;j<=mNbPhi;j++) {
			phi = ((float) j/mNbPhi - 1./2.)*mPhiAngle; 
			if(mInside) {
				*ptr = Vector3(-cos(lambda)*sin(phi),-sin(lambda),-cos(lambda)*cos(phi));
			} else {
				*ptr = Vector3(cos(lambda)*sin(phi),sin(lambda),cos(lambda)*cos(phi));
			}
	    	ptr ++;
		}
	}
}

void RoundCornerMesh::createTexCoords(GLfloat *ptr) {
	float lambda, phi;
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = (float) i/mNbLambda ; 
		for(int j=0;j<=mNbPhi;j++) {
			phi = (float) j/mNbPhi; 
			*ptr = mPhiTexBegin + phi /(mPhiTexEnd-mPhiTexBegin);
	    	ptr ++;
			*ptr = mLambdaTexBegin + lambda / (mLambdaTexEnd -mLambdaTexBegin);
	    	ptr ++;
		}
	}
}

void RoundCornerMesh::createFogCoords(GLfloat *ptr) {

}
	
void RoundCornerMesh::createTangents(Vector3 *ptr) {
	float lambda, phi;
	float lambdaMin = M_PI /2. - mLambdaAngle;	
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/mNbLambda)*mLambdaAngle + lambdaMin; 
		for(int j=0;j<=mNbPhi;j++) {
			phi = ((float) j/mNbPhi - 1./2.)*mPhiAngle; 
			if(mInside) {
				*ptr = Vector3(-cos(phi),0.,sin(phi));
			} else {
				*ptr = Vector3(cos(phi),0.,-sin(phi));
			}
	    	ptr ++;
		}	
	}
}

		
void RoundCornerMesh::createIndices(int numArray, GLuint *ptr) {
	if(mInside) {	
		for(int j=0;j<=mNbPhi;j++) {
			*ptr = numArray * (mNbPhi+1) + j;
			ptr ++;
			*ptr = (numArray+1) * (mNbPhi+1) + j;
			ptr ++;
		}
		*ptr = numArray * (mNbPhi+1);
		ptr ++;
		*ptr = (numArray+1) * (mNbPhi+1);
		ptr ++;

	} else {
		for(int j=0;j<=mNbPhi;j++) {
			*ptr = (numArray+1) * (mNbPhi+1) + j;
			ptr ++;			
			*ptr = numArray * (mNbPhi+1) + j;
			ptr ++;
		}	
		*ptr = (numArray+1) * (mNbPhi+1);
		ptr ++;			
		*ptr = numArray * (mNbPhi+1);
		ptr ++;
	}
}
