/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "sphere.h"
#include "../newtEngine/scene.h"


SphereMesh::SphereMesh(float radius, int nbPhi, int nbLambda, bool inside):
	Mesh("sphere", false, true, false, true), mRadius(radius), mInside(inside) {
			
	float meshLevel = scene->getGraphicOptions()->getMeshLevel();
	float value;
		
	value = (float) nbPhi * meshLevel;
	if(value < 4) {
		mNbPhi = 4;
	} else {
		mNbPhi = (int) value;
	}

	value = (float) nbLambda * meshLevel;
	if(value < 4) {
		mNbLambda = 4;
	} else {
		mNbLambda = (int) value;
	}			
}
	
SphereMesh::~SphereMesh() {
}
	

void SphereMesh::initFirstParams() {
	geometry.nbVertices = (mNbPhi + 1) * ( mNbLambda + 1);
	
	geometry.nbIndexArrays = mNbLambda;
	geometry.primType = new GLenum[mNbLambda];

	geometry.primType[0] = GL_TRIANGLES;
	for(int i=1;i<mNbLambda-1;i++) {
		geometry.primType[i] = GL_QUAD_STRIP;
	}
	geometry.primType[mNbLambda-1] = GL_TRIANGLES;	
	
	geometry.nbIndices = new int[mNbLambda];
	
	geometry.nbIndices[0] = 3 * mNbPhi;
	for(int i=1;i<mNbLambda-1;i++) {
		geometry.nbIndices[i] = 2 * (mNbPhi+1);
	}	
	geometry.nbIndices[mNbLambda-1] = 3 * mNbPhi;

}
		
void SphereMesh::createVertices(Vector3 *ptr) {
	float lambda, phi;
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/mNbLambda  - 1./2.)*M_PI; 
		for(int j=0;j<mNbPhi;j++) {
			phi = ((float) j/mNbPhi)*2.*M_PI; 
			*ptr = mRadius * Vector3(cos(lambda)*sin(phi),sin(lambda),cos(lambda)*cos(phi));
	    	ptr ++;
		}
		*ptr = mRadius * Vector3(0.,sin(lambda),cos(lambda));
		ptr ++;
	}
}

void SphereMesh::createColors(Color *ptr) {

}

void SphereMesh::createNormals(Vector3 *ptr) {
	float lambda, phi;
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = ((float) i/ (float) mNbLambda  - 1./2.)*M_PI; 
		for(int j=0;j<mNbPhi;j++) {
			phi = ((float) j/ (float) mNbPhi)*2.*M_PI; 
			if(mInside) {
				*ptr = Vector3(-cos(lambda)*sin(phi),-sin(lambda),-cos(lambda)*cos(phi));
			} else {
				*ptr = Vector3(cos(lambda)*sin(phi),sin(lambda),cos(lambda)*cos(phi));
			}
	    	ptr ++;
		}
		if(mInside) {	
			*ptr = Vector3(0.,-sin(lambda),-cos(lambda));
		} else {
			*ptr = Vector3(0.,sin(lambda),cos(lambda));
		}		
		ptr ++;
	}
}

void SphereMesh::createTexCoords(GLfloat *ptr) {
	float lambda, phi;
	
	for(int i=0;i<=mNbLambda;i++) {
		lambda = (float) i/mNbLambda ; 
		for(int j=0;j<mNbPhi;j++) {
			phi = (float) j/mNbPhi; 
			*ptr = phi;
	    	ptr ++;
			*ptr = lambda;
	    	ptr ++;
		}
		*ptr = 1.;
		ptr ++;
		*ptr = (float) i/mNbLambda;
		ptr ++;
	}
}

void SphereMesh::createFogCoords(GLfloat *ptr) {

}
	
void SphereMesh::createTangents(Vector3 *ptr) {
	float lambda, phi;
	
	for(int j=0;j<=mNbPhi;j++) {
		phi = (((float) j + 0.5)/(float) mNbPhi)*2.*M_PI; 
		if(mInside) {
			*ptr = Vector3(-cos(phi),0.,sin(phi));
		} else {
			*ptr = Vector3(cos(phi),0.,-sin(phi));
		}
		ptr ++;
	}
	
	for(int i=1;i<mNbLambda;i++) {
		for(int j=0;j<mNbPhi;j++) {
			phi = ((float) j/(float) mNbPhi)*2.*M_PI; 
			if(mInside) {
				*ptr = Vector3(-cos(phi),0.,sin(phi));
			} else {
				*ptr = Vector3(cos(phi),0.,-sin(phi));
			}
	    	ptr ++;
		}
		
		if(mInside) {	
			*ptr = Vector3(-1.,0.,0.);
		} else {
			*ptr = Vector3(1.,0.,0.);
		}		
		ptr ++;	
	}
	
	for(int j=0;j<=mNbPhi;j++) {
		phi = (((float) j + 0.5)/(float) mNbPhi)*2.*M_PI; 
		if(mInside) {
			*ptr = Vector3(-cos(phi),0.,sin(phi));
		} else {
			*ptr = Vector3(cos(phi),0.,-sin(phi));
		}
		ptr ++;
	}
}

		
void SphereMesh::createIndices(int numArray, GLuint *ptr) {
	
	if(numArray==0) {
		
		if(mInside) {	
			for(int j=0;j<mNbPhi;j++) {	
				*ptr = (mNbPhi+1) + j;
				ptr ++;				
				*ptr = (mNbPhi+1) + j+1;
				ptr ++;	
				*ptr = j;
				ptr ++;					
			}
		} else {
			for(int j=0;j<mNbPhi;j++) {	
				*ptr = (mNbPhi+1) + j;
				ptr ++;					
				*ptr = j;
				ptr ++;	
				*ptr = (mNbPhi+1) + j+1;
				ptr ++;					
			}			
		}
		
	} else if(numArray == mNbLambda-1) {
		
		if(mInside) {	
			for(int j=0;j<mNbPhi;j++) {	
				*ptr = numArray * (mNbPhi+1) + j;
				ptr ++;	
				*ptr = (numArray+1) * (mNbPhi+1) + j;
				ptr ++;					
				*ptr = numArray * (mNbPhi+1) + j+1;
				ptr ++;					
			}
		} else {
			for(int j=0;j<mNbPhi;j++) {	
				*ptr = numArray * (mNbPhi+1) + j;
				ptr ++;	
				*ptr = numArray * (mNbPhi+1) + j+1;
				ptr ++;					
				*ptr = (numArray+1) * (mNbPhi+1) + j;
				ptr ++;			
			}			
		}
	
	} else {
		
		if(mInside) {	
			for(int j=0;j<=mNbPhi;j++) {
				*ptr = numArray * (mNbPhi+1) + j;
				ptr ++;
				*ptr = (numArray+1) * (mNbPhi+1) + j;
				ptr ++;
			}
			*ptr = numArray * (mNbPhi+1);
			ptr ++;
			*ptr = (numArray+1) * (mNbPhi+1);
			ptr ++;

		} else {
			for(int j=0;j<=mNbPhi;j++) {
				*ptr = (numArray+1) * (mNbPhi+1) + j;
				ptr ++;			
				*ptr = numArray * (mNbPhi+1) + j;
				ptr ++;
			}	
			*ptr = (numArray+1) * (mNbPhi+1);
			ptr ++;			
			*ptr = numArray * (mNbPhi+1);
			ptr ++;
		}
	}
}


SphereWithPosition::SphereWithPosition(float radius, int nbPhi, int nbLambda, bool inside, Material *material, Vector3 center, Vector3 rotAxis, float angle):
EntityWithPosition(material, new SphereMesh(radius, nbPhi, nbLambda, inside), true, center, rotAxis, angle) {
	
}

SphereWithPosition::~SphereWithPosition() {
	
}


